//interface.cc: implementation file for sim-crowbar interface
//state machine code

#include "interface.h"
#include "block_array.h"
#include "signal.h"
#include <iostream.h>
#include "string.h"

//extern CROWBAR_class* chip;

///////////////////////////////////////////////////////////////////////////////
//Parser routines

state_type next_state(token_type token, token_data data, state_type old_state)
{
  extern CROWBAR_class* chip;
  state_type new_state = old_state;
  parse_error_type error_code;
  bool lambda = true; //lambda transitions between states
  voltage_type voltage;
  optical_type optic;
 
  while(lambda == true)
    { 
      switch(new_state)
	{
	case START_STATE:
	  lines = 0;
	  chip->Clear();
	  print_intro();
	  new_state = PROMPT_STATE;
	  break;
	case PROMPT_STATE:
	  prompt_for_input(lines);
	  lines++;
	  new_state = WAIT_FOR_COMMAND;
	  break;
	case WAIT_FOR_COMMAND:
	  switch(token)
	    {
	    case ASSIGN_PIN:
	      new_state = WAIT_FOR_PIN;
	      switch(data.command)
		{
		case 'h': voltage = High; break;
		case 'l': voltage = Low; break;
		default: voltage = Floating;
		} //switch
	      break;
	    case ASSIGN_OPTICAL:
	      new_state = WAIT_FOR_OPTIC;
	      break;
	    case QUIT:
	      new_state = CONFIRM_QUIT;
	      break;
	    case PRINT:
	      new_state = WAIT_FOR_OPTION;
	      break;
	    case SIMULATE:
	      new_state = WAIT_FOR_GATE_DELAYS;
	      break;
	    case HELP:
	      new_state = HELP_STATE;
	      break;
	    case NAME:
	    case POS_INT:
	    case YES:
	    case NO:
	    case ERROR:
	      error_code = UNDEFINED_COMMAND;
	      new_state = ERROR_STATE;
	      break;
	    case END:
	    default:
	      new_state = WAIT_FOR_COMMAND;
	    } //switch
	  break;
	case WAIT_FOR_PIN:
	  switch(token)
	    {
	    case NAME:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin(data.name, voltage);
	      break;
	    case POS_INT:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin((pin_name_type)data.ival, voltage);
	      break;
	    default:
	      error_code = INVALID_PIN;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_ANOTHER_PIN:
	  switch(token)
	    {
	    case NAME:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin(data.name, voltage);
	      break;
	    case POS_INT:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin((pin_name_type)data.ival, voltage);
	      break;
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_PIN;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_OPTIC:
	  switch(token)
	    {
	    case POS_INT:
	      optic = (optical_type)data.ival;
	      new_state = WAIT_FOR_SENSOR;
	      break;
	    default:
	      error_code = INVALID_OPTIC;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_GATE_DELAYS:
	  switch(token)
	    {
	    case POS_INT:
	      check_and_simulate(data.ival);
	      new_state = WAIT_FOR_EOL;
	      break;
	    default:
	      error_code = INVALID_GATE_DELAYS;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_SENSOR:
	  switch(token)
	    {
	    case NAME:
	      check_and_assign_optical(data.name, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    case POS_INT:
	      check_and_assign_optical(data.ival, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    case END: 
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_NAME;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_ANOTHER_SENSOR:
	  switch(token)
	    {
	    case NAME:
	      check_and_assign_optical(data.name, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    case POS_INT:
	      check_and_assign_optical(data.ival, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    default:
	      error_code = INVALID_NAME;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_OPTION:
	  switch(token)
	    {
	    case NAME:
	      check_and_print_data(data.name);
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_OPTION;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case HELP_STATE:
	  print_help();
	  new_state = PROMPT_STATE;
	  break;
	case WAIT_FOR_EOL:
	  switch(token)
	    {
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = EOL_EXPECTED;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case ERROR_STATE:
	  //chew up the rest of the line
          print_error(error_code, lines);
	  switch(token)
	    {
	    case END:
	      new_state = PROMPT_STATE; //that was it
	      break;
	    default:
	      new_state = DISCARD_LINE; //other stuff
	    }
	  break;
	case DISCARD_LINE:
	  switch(token)
	    {
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      ;
	    } //switch
	  break;
	case CONFIRM_QUIT:
          prompt_for_confirmation();
	  new_state = WAIT_FOR_RESPONSE;
	  break;
	case WAIT_FOR_RESPONSE:
	  switch(token)
	    {
	    case YES:
	      new_state = END_STATE;
	      break;
	    case NO:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_RESPONSE;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	default:
	  new_state = PROMPT_STATE;
	} //switch old_state
 
      switch(new_state)
	{ 
	case WAIT_FOR_COMMAND:
	case WAIT_FOR_PIN:
	case WAIT_FOR_ANOTHER_PIN:
	case WAIT_FOR_OPTIC:
	case WAIT_FOR_GATE_DELAYS:
	case WAIT_FOR_SENSOR:
	case WAIT_FOR_ANOTHER_SENSOR:
	case WAIT_FOR_OPTION:
	case WAIT_FOR_RESPONSE:
	case WAIT_FOR_EOL:
	case DISCARD_LINE:
        case END_STATE:
	  lambda = false;
	  break;
	default:
	  lambda = true;
	} //switch
    }//while
  return new_state;
} //state_machine


void print_intro()
{
 cout<<"Welcome to Sim-Crowbar:"<<endl;
 cout<<"The interactive simulator for the Crowbar Prototype"<<endl;
 cout<<"Jason Cantin, 2001"<<endl;
} //print_intro


void print_error(parse_error_type code, counter_type line_number)
{
  cerr<<"***Parse error: line "<<line_number<<", ";
  switch(code)
    {
    case UNDEFINED_COMMAND:
      cerr<<"Undefined command"<<endl;
      break;
    case INVALID_PIN:
      cerr<<"Invalid pin argument"<<endl;
      break;
    case INVALID_OPTIC:
      cerr<<"Invalid optical intensity"<<endl;
      break;
    case INVALID_GATE_DELAYS:
      cerr<<"Invalid gate delays"<<endl;
      break;
    case INVALID_NAME:
      cerr<<"Invalid name"<<endl;
      break;
    case INVALID_OPTION:
      cerr<<"Invalid print option"<<endl;
      break;
    case EOL_EXPECTED:
      cerr<<"Characters found where end-of-line expected"<<endl;
      break;
    case INVALID_RESPONSE:
      cerr<<"Invalid response"<<endl;
      break;
    default:
      cerr<<"Error type unknown"<<endl;
    } //switch
  cerr<<"\tType 'help' for usage"<<endl;
} //print_error


void prompt_for_input(counter_type line_number)
{
   cout<<"sim-crowbar("<<line_number<<"):";
} //prompt_for_input


void prompt_for_confirmation()
{
 cout<<"Quitting: Are you sure? (y/n)\n";
} //prompt_for_confirmation


void print_help()
{	
  cout<<"***Valid commands for Sim-Crowbar***"<<endl;
  cout<<endl;
  cout<<"[ l | h | x ] [<pin_name> | <pin_number>]"<<endl;
  cout<<"\tSet corresponding pin voltage low, high, or floating."<<endl;
  cout<<endl;
  cout<<"o <intensity> <sensor_name>"<<endl;
  cout<<"\tApply optical energy to named sensor input."<<endl;
  cout<<"\tIntensities range from 0-7 in this implementation."<<endl;
  cout<<endl;
  cout<<"s <gate-delays>"<<endl;
  cout<<"\tSimulates logic for (at most) specified number of delays."<<endl;
  cout<<"\tStops if signals converge before that time."<<endl;
  cout<<endl;
  cout<<"p [pins | config | image | flops | block_outputs]"<<endl;
  cout<<"\tPrints indicated information about state of chip."<<endl;
  cout<<endl;
  cout<<"help"<<endl;
  cout<<"\tPrints this message."<<endl;
  cout<<endl;
  cout<<"q"<<endl;
  cout<<"\tQuits the simulator."<<endl;
  cout<<endl;
  cout<<"Comments begin with a vertical bar ('|'), and terminate"<<endl;
  cout<<"at the end of the line."<<endl; 
  cout<<endl;
} //print_help

///////////////////////////////////////////////////////////////////////////////
//Semantic routines

void check_and_assign_pin(char* name, voltage_type voltage)
{
  pin_name_type pin_name = String_To_Pin_Name(name);
  check_and_assign_pin(pin_name, voltage);
} //check_and_assign_pin


void check_and_assign_pin(pin_name_type pin_name, voltage_type voltage)
{
  extern CROWBAR_class* chip;
  semantic_error_type error;
  if((pin_name <= UNDEFINED_PIN)||(pin_name >= PINS))
    {
      error = PIN_NUMBER_OUT_OF_RANGE;
      print_error(error, lines);
    } //if
  else
    {
      //name okay, assign the voltage
      chip->Apply_Voltage_To_Pin(pin_name, voltage);
    } //else
} //check_and_assign_pin


void check_and_print_data(char* option)
{
  extern CROWBAR_class* chip;
  semantic_error_type error;
  if(strcasecmp("pins", option)==0)
    chip->Print_Pin_Values();
  else
    if(strcasecmp("config", option)==0)
      chip->Print_Config();
    else
      if(strcasecmp("flops", option)==0)
	chip->Print_Flop_Outputs();
      else
	if(strcasecmp("image", option)==0)
	  chip->Print_Image();
	else
	  if(strcasecmp("block_outputs", option)==0)
	    chip->Print_Block_Outputs();
	  else
	    {
	      error = UNKNOWN_OPTION;
	      print_error(error, lines);
	    }
} //check_and_print_data


void check_and_simulate(unsigned int gate_delays)
{
  extern CROWBAR_class* chip;
  chip->Update(gate_delays);
} //check_and_simulate


void check_and_assign_optical(char* name, optical_type optic)
{
  //check name
  semantic_error_type error;
  csb_name_type number = String_To_CSB_Name(name);
  if(number == UNDEFINED_CSB)
    {
      error = CSB_NUMBER_OUT_OF_RANGE;
      print_error(error, lines);
    }
  else
    {
      check_and_assign_optical(number, optic);
    } //else
}  //check_and_assign_optical


void check_and_assign_optical(int num, optical_type optic)
{
  semantic_error_type error;
  //first check value of intensity
  if((optic<0)||(optic>7))
    {
      error = INTENSITY_OUT_OF_RANGE;
      print_error(error, lines);
    } //if
  else
    {
      chip->Apply_Light_To_Sensor(num, optic);
    }
} //check_and_assign_optical


void print_error(semantic_error_type code, counter_type line_number)
{
  cerr<<"***Semantic error: line "<<line_number<<", ";
  switch(code)
    {
    case PIN_NUMBER_OUT_OF_RANGE:
      cerr<<"Unknown Pin"<<endl;
      break;
    case INTENSITY_OUT_OF_RANGE:
      cerr<<"Intensity out of range, must be 0-7"<<endl;
      break;
    case UNKNOWN_OPTION:
      cerr<<"Unknown command option specified"<<endl;
      break;
    case CSB_NUMBER_OUT_OF_RANGE:
      cerr<<"Unknown Sensor"<<endl;
      break;
    default:
      cerr<<"Error type unknown"<<endl;
    } //switch
  cerr<<"\tType 'help' for usage"<<endl;
} //print_error


///////////////////////////////////////////////////////////////////////////////
//End implementation file







